/**
 * Copyright 2013-2015, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * @typechecks static-only
 * @providesModule ReactServerAsyncRendering
 */
'use strict';

var _get = function get(_x, _x2, _x3) { var _again = true; _function: while (_again) { var object = _x, property = _x2, receiver = _x3; desc = parent = getter = undefined; _again = false; if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { _x = parent; _x2 = property; _x3 = receiver; _again = true; continue _function; } } else if ('value' in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } } };

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var ReactDefaultBatchingStrategy = require('./ReactDefaultBatchingStrategy');
var ReactElement = require('./ReactElement');
var ReactInstanceHandles = require('./ReactInstanceHandles');
var ReactMarkupChecksum = require('./ReactMarkupChecksum');
var ReactServerBatchingStrategy = require('./ReactServerBatchingStrategy');
var ReactServerRenderingTransaction = require('./ReactServerRenderingTransaction');
var ReactUpdates = require('./ReactUpdates');

var emptyObject = require('fbjs/lib/emptyObject');
var instantiateReactComponent = require('./instantiateReactComponent');
var invariant = require('fbjs/lib/invariant');

var rollingAdler32 = require('./rollingAdler32');
var stream = require('stream');

var GeneratorBuffer = (function () {
  function GeneratorBuffer(initialChunkSize) {
    _classCallCheck(this, GeneratorBuffer);

    this.chunkSize = initialChunkSize;
    this.buffer = "";
  }

  _createClass(GeneratorBuffer, [{
    key: 'addAndYieldBuffer',
    value: function* addAndYieldBuffer(text) {
      // TODO: this is naive and could go over the chunk size.
      this.buffer += text;
      if (this.buffer.length > this.chunkSize) {
        // TODO: get an update of new chunk size.
        yield this.buffer;
        this.buffer = "";
      }
    }

    // returns a boolean about whether to yield. should be used like:
    //    if (buf.add("foo")) yield * buf.yieldBuffer();
  }, {
    key: 'add',
    value: function add(text) {
      this.buffer += text;
      return this.buffer.length > this.chunkSize;
    }
  }, {
    key: 'yieldBuffer',
    value: function* yieldBuffer() {
      yield this.buffer;
      this.buffer = "";
    }
  }]);

  return GeneratorBuffer;
})();

var IteratorReadable = (function (_stream$Readable) {
  _inherits(IteratorReadable, _stream$Readable);

  function IteratorReadable(iterator, options) {
    _classCallCheck(this, IteratorReadable);

    _get(Object.getPrototypeOf(IteratorReadable.prototype), 'constructor', this).call(this, options);
    this.iterator = iterator;
  }

  // this is a pass through stream that can calculate the hash that is used to
  // checksum react server-rendered elements.

  _createClass(IteratorReadable, [{
    key: '_read',
    value: function _read(n) {
      var countRead = 0;

      var done,
          value,
          buffer = "";

      while ((_iterator$next = this.iterator.next(), done = _iterator$next.done, value = _iterator$next.value, _iterator$next)) {
        var _iterator$next;

        if (done) {
          this.push(buffer);
          this.push(null);
          return;
        } else {
          buffer += value;
          if (buffer.length >= n) {
            this.push(buffer);
            return;
          }
        }
      }
    }
  }]);

  return IteratorReadable;
})(stream.Readable);

var Adler32Stream = (function (_stream$Transform) {
  _inherits(Adler32Stream, _stream$Transform);

  function Adler32Stream(options) {
    var _this = this;

    _classCallCheck(this, Adler32Stream);

    _get(Object.getPrototypeOf(Adler32Stream.prototype), 'constructor', this).call(this, options);
    this.rollingHash = rollingAdler32("");
    this.on("end", function () {
      _this.done = true;
    });
  }

  /**
   * @param {ReactElement} element
   * @return {string} the HTML markup
   */

  _createClass(Adler32Stream, [{
    key: '_transform',
    value: function _transform(chunk, encoding, next) {
      this.rollingHash = rollingAdler32(chunk.toString("utf-8"), this.rollingHash);
      this.push(chunk);
      next();
    }

    // returns a promise of a hash that resolves when this readable piped into this is finished.
  }, {
    key: 'hash',
    get: function () {
      var _this2 = this;

      if (this.done) {
        return Promise.resolve(this.rollingHash.hash());
      }
      return new Promise(function (resolve, reject) {
        _this2.on("end", function () {
          resolve(_this2.rollingHash.hash());
        });
      });
    }
  }]);

  return Adler32Stream;
})(stream.Transform);

function renderToStringStream(element) {
  !ReactElement.isValidElement(element) ? process.env.NODE_ENV !== 'production' ? invariant(false, 'renderToStringStream(): You must pass a valid ReactElement.') : invariant(false) : undefined;

  var transaction;

  ReactUpdates.injection.injectBatchingStrategy(ReactServerBatchingStrategy);

  var id = ReactInstanceHandles.createReactRootID();
  transaction = ReactServerRenderingTransaction.getPooled(false);

  var result = transaction.perform(function () {
    var componentInstance = instantiateReactComponent(element, null);
    return renderToStringIterator(componentInstance, id, transaction, emptyObject);
  }, null);
  var readable = new IteratorReadable(result);
  readable.on("end", function () {
    ReactServerRenderingTransaction.release(transaction);
    // Revert to the DOM batching strategy since these two renderers
    // currently share these stateful modules.
    ReactUpdates.injection.injectBatchingStrategy(ReactDefaultBatchingStrategy);
  });

  // since Adler32Stream has a .hash property, this automagically adds that property to the result.
  return readable.pipe(new Adler32Stream());
}

function* renderToStringIterator(componentInstance, id, transaction, context) {
  // TODO: how to set this up? what number should this be?
  var buf = new GeneratorBuffer(16 * 1024);
  yield* componentInstance.mountComponentAsync(id, transaction, context, buf);
  yield* buf.yieldBuffer();
}

/**
 * @param {ReactElement} element
 * @return {string} the HTML markup, without the extra React ID and checksum
 * (for generating static pages)
 */
function renderToStaticMarkupStream(element) {
  !ReactElement.isValidElement(element) ? process.env.NODE_ENV !== 'production' ? invariant(false, 'renderToStaticMarkupStream(): You must pass a valid ReactElement.') : invariant(false) : undefined;

  var transaction;

  ReactUpdates.injection.injectBatchingStrategy(ReactServerBatchingStrategy);

  var id = ReactInstanceHandles.createReactRootID();
  transaction = ReactServerRenderingTransaction.getPooled(true);

  var result = transaction.perform(function () {
    var componentInstance = instantiateReactComponent(element, null);
    return renderToStringIterator(componentInstance, id, transaction, emptyObject);
  }, null);

  var readable = new IteratorReadable(result);
  readable.on("end", function () {
    ReactServerRenderingTransaction.release(transaction);
    // Revert to the DOM batching strategy since these two renderers
    // currently share these stateful modules.
    ReactUpdates.injection.injectBatchingStrategy(ReactDefaultBatchingStrategy);
  });

  return readable;
}

module.exports = {
  renderToStringStream: renderToStringStream,
  renderToStaticMarkupStream: renderToStaticMarkupStream
};